<?php
require_once('../inc/e2v_system.php');

// an initial check that the post seems to be coming from SL...
$shard = $_SERVER['HTTP_X_SECONDLIFE_SHARD'];
if ( $shard != 'Production' )
{
	exit;
}

// check that the password is valid
if ( $_POST['pw'] != OBJECT_PASSWORD )
{
	exit;
}

header("content-type: text/plain; charset=utf-8");
date_default_timezone_set('America/Los_Angeles');

// misc const
define('PAYMENT_LINDEN', 'L');
define('PAYMENT_CREDIT', 'C');


// our required files
require_once('../inc/e2v_db.php');
require_once('../inc/e2v_utils.php');

// clean up all posted variables
DBConnect();
CleanPost();

$ownerKey = $_SERVER['HTTP_X_SECONDLIFE_OWNER_KEY'];
$storeID = GetStoreID( $ownerKey );
if ( $storeID == 0 )
{
	E2VError('You are not a registered store owner.');
	exit;
}

// our reply
$resp = array();

// grab remaining fields
$custKey = $_POST['custkey'];
$custName = rawurldecode( $_POST['custname'] );
$cost = CleanInt( 'price' );
$paymentMethod = $_POST['method'];
$item = rawurldecode( $_POST['item'] );
$reward = CleanInt( 'credit' );
$recipKey = $_POST['recipkey'];
$recipName = rawurldecode( $_POST['recipname'] );

if ( !isset($recipKey) ) $recipKey = '';
if ( !isset($recipName) ) $recipName = '';

// use PHP dates 
$date = time();

// make sure we have a valid sales type
if ( !in_array( $paymentMethod, array(PAYMENT_LINDEN, PAYMENT_CREDIT) ) )
{
	E2VError( 'Invalid payment method (' . $paymentMethod . ')');
	exit;
}

// for a credits sale, make sure the number of credits required is valid
if ( ( $paymentMethod == PAYMENT_CREDIT ) && ( $cost == 0 ) )
{
	E2VError( 'Invalid credit required' );
	exit;
}

// we'll need this customer's current credit balance
$balance = GetCredit( $custKey, $storeID );

// from here on, we'll use the $resp array for return values, so
// put in the standard values
$resp[FIELD_METHOD] = $paymentMethod;
$resp[FIELD_BALANCE] = $balance;
$resp[FIELD_CUSTKEY] = $custKey;

// adjust the credit balance for rewards
$newBalance = $balance;
if ( ( $paymentMethod == PAYMENT_LINDEN ) && ( $reward > 0 ) )
	$newBalance = $newBalance + $reward;

// need to update the balance?
if ( $newBalance != $balance )
{
	SetCredit( $custKey, $custName, $storeID, $newBalance );
	$resp[FIELD_REWARDED] = $reward;
	$resp[FIELD_BALANCE] = $newBalance;
}

// get, and clean up, the SL region (remove the global co-ords)
$region = CleanRegion( $_SERVER['HTTP_X_SECONDLIFE_REGION'] );

// record the transaction
$sql = <<<SQL
INSERT INTO sales 
 VALUES (0, $storeID, "$region", $date, "$custKey", "$custName", "$item", "$paymentMethod", $cost, $reward, "$recipKey", "$recipName")
SQL;

$result = run_mysql_query( $sql );
if ( mysql_affected_rows() == 1 )
{
	$resp[FIELD_STATUS] = STATUS_OK;
}
else
{
	$resp[FIELD_STATUS] = STATUS_ERR;
	$resp[FIELD_ERROR] = mysql_error();
}

LSLReply( $resp );

