<?php

// make sure we're authorized to be loading this file
session_start();
$storeID = $_SESSION['storeid'];
$ownerKey = $_SESSION['ownerkey'];

if ( !isset($storeID) ) {
	exit;
}

require_once('../inc/e2v_system.php');
require_once('../inc/e2v_db.php');
require_once('../inc/e2v_utils.php');

// commands we handle...
define('CMD_LOOKUP', 'lookup');
define('CMD_SUMMARY', 'summary');
define('CMD_SETCREDIT', 'setcredit');

DBConnect();
CleanPost();

$resp = array();

$command = strtolower( filter_var($_POST['command'], FILTER_SANITIZE_STRING) );
if ( !in_array( $command, array(CMD_LOOKUP, CMD_SUMMARY, CMD_SETCREDIT) ) )
{
	$resp[FIELD_STATUS] = STATUS_ERR;
	$resp[FIELD_ERROR] = 'Unrecognized command.';
	print json_encode( $resp );
	exit;
}

// CMD_LOOKUP
// Searches for matching avatar names for autocomplete feature
if ( $command == CMD_LOOKUP )
{
	$query = $_POST['term'];
	
	$candidates = array();
	
	if ( strlen($query) >= 3 )
	{
		$sql = <<<SQL
SELECT DISTINCT(custname),custkey FROM sales
  WHERE (storeid=$storeID) AND (custname LIKE "$query%")
  LIMIT 12
SQL;
	
		$result = mysql_query( $sql );
		while ( $row = mysql_fetch_assoc( $result ) )
		{
			$item = array();
			$item['label'] = $row['custname'];
			$item['value'] = $row['custname'];
			$item['uuid'] = $row['custkey'];
			
			$candidates[] = $item;
		}
	}
	
	print json_encode($candidates);
	exit;	
}


// CMD_SUMMARY
// Gathers spending and transaction info for a customer
if ( $command == CMD_SUMMARY )
{
	// see if we have the UUID to look up
	$uuid = $_POST['uuid'];
	$uuid = ( isset($uuid) && $uuid != '' ) ? $uuid : '';
	
	if ( $uuid == '' || (!isValidGuid($uuid)) )
	{
		$resp[FIELD_STATUS] = STATUS_ERR;
		$resp[FIELD_ERROR] = 'Bad parameters';
		print json_encode( $resp );
		exit;
	}
	
	// otherwise onward...
	$avatar = array();
	$financials = array();
	$transactions = array();

	$resp[FIELD_STATUS] = STATUS_OK;
	
	$financials['L'] = 0; // lindens spent
	$financials['C'] = 0; // credit spent
	$financials['earned'] = 0; // credit earned
	$financials['balance'] = 0;  // credit balance

	$avatar['name'] = '';
	$avatar['uuid'] = $uuid;
	$avatar['link'] = '';	

	// gather the transactions
	$sql = <<<SQL
SELECT id,FROM_UNIXTIME(date) AS tdate, region, custname, custkey, recipname, recipkey,item, cost, paytype, reward
FROM sales
WHERE (storeid=$storeID) AND (custkey="$uuid")
ORDER BY date DESC;
SQL;

	$result = mysql_query($sql);
	
	while ( $row = mysql_fetch_assoc($result) )
	{
		$cells = array();
		
		$cells[] = $row['tdate'];
		$cells[] = $row['region'];
		$cells[] = $row['item'];
		
		// get the recipient
		$recipCell = '';
		$recipKey = $row['recipkey'];
		$recipName = $row['recipname'];
		if ( strlen($recipKey) > 0 && strlen($recipName) > 0 )
		{
			$recipCell = '<a target="_blank" title="' . $recipKey . '" href="http://world.secondlife.com/resident/';
			$recipCell .= $recipKey . '">' . $recipName . '</a>';
		}
		
		$cells[] = $recipCell;
		$cells[] = $row['cost'];
		$cells[] = $row['paytype'];
		$cells[] = $row['reward'];
		
		// the redeliver checksum is the 9th, 11th, 6th and 3rd characters of the hash of
		// the ownerKey + customerKey + 'xx1332'
		$hash = sha1( $ownerKey . $uuid . OBJECT_PASSWORD );
		$checksum = substr($hash, 9, 1) . substr($hash, 11, 1) . substr($hash, 6, 1) . substr($hash, 3, 1);
		
		// construct the redelivery column
		$cells[] = '<a class="redlink" data-transid="' . $row['id'] . '-' . $checksum . '" href="#">redeliver</a>';
		

		$transactions[] = $cells;
		
		// make sure we have this...
		$avatar['name'] = $row['custname'];
		
		// running financial totals...
		$financials[ $row['paytype'] ] += intval( $row['cost'] );
		$financials['earned'] += intval( $row['reward'] );
	}

	// credit balance
	$sql = <<<SQL
SELECT balance FROM credits
  WHERE (storeid=$storeID) AND (custkey="$uuid");	
SQL;
	
	$result = run_mysql_query($sql);
	if ( $row = mysql_fetch_assoc($result) )
	{
		$balance = $row['balance'];
		if ( isset($balance) && $balance != '' )
		  $financials['balance'] = intval($balance);
	}

	// make the profile link
	$link = '<a target="_blank" title="' . $uuid . '" href="http://world.secondlife.com/resident/';
	$link .= $uuid . '">View Profile</a>';
	$avatar['link'] = $link;

	// put together the response...
	$resp['avatar'] = $avatar;
	$resp['financials'] = $financials;
	$resp['transactions'] = $transactions;	
  print json_encode( $resp );
	
	exit;
}


// CMD_SETCREDIT
// Updates the user's credit value to what's specified. 
// Note that we might be creating a new record.
if ( $command == CMD_SETCREDIT )
{
	// grab the parms
	$uuid = $_POST['uuid'];
	$name = $_POST['name'];
	$balance = $_POST['balance'];
	
	if ( SetCredit( $uuid, $name, $storeID, $balance ) )
	{
		$resp[FIELD_STATUS] = STATUS_OK;
	}
	else
	{
		$resp[FIELD_STATUS] = STATUS_ERR;
		$resp[FIELD_ERROR] = mysql_error();
	}

	print json_encode( $resp );
	exit;	
}

